package cz.cuni.amis.pogamut.ut2004.lectures.tagbot;

import java.util.logging.Level;

import cz.cuni.amis.pogamut.base.utils.guice.AgentScoped;
import cz.cuni.amis.pogamut.base3d.worldview.object.Location;
import cz.cuni.amis.pogamut.unreal.communication.messages.UnrealId;
import cz.cuni.amis.pogamut.ut2004.communication.messages.gbcommands.Initialize;
import cz.cuni.amis.pogamut.ut2004.communication.messages.gbinfomessages.ConfigChange;
import cz.cuni.amis.pogamut.ut2004.communication.messages.gbinfomessages.ControlMessage;
import cz.cuni.amis.pogamut.ut2004.communication.messages.gbinfomessages.GameInfo;
import cz.cuni.amis.pogamut.ut2004.communication.messages.gbinfomessages.InitedMessage;
import cz.cuni.amis.pogamut.ut2004.communication.messages.gbinfomessages.Player;
import cz.cuni.amis.pogamut.ut2004.communication.messages.gbinfomessages.Self;
import cz.cuni.amis.pogamut.ut2004.factory.guice.remoteagent.UT2004ServerFactory;
import cz.cuni.amis.pogamut.ut2004.tag.bot.TagEvents;
import cz.cuni.amis.pogamut.ut2004.tag.bot.UT2004BotTagController;
import cz.cuni.amis.pogamut.ut2004.tag.protocol.messages.TagPlayerStatusChanged;
import cz.cuni.amis.pogamut.ut2004.tag.server.UT2004TagServer;
import cz.cuni.amis.pogamut.ut2004.tag.server.UT2004TagServerModule;
import cz.cuni.amis.pogamut.ut2004.utils.UT2004BotRunner;
import cz.cuni.amis.pogamut.ut2004.utils.UT2004ServerRunner;
import cz.cuni.amis.utils.exception.PogamutException;
import java.util.Iterator;

/**
 * Example of the bot playing TAG-GAME according to {@link ControlMessage}s from {@link UT2004TagServer}.
 * <p><p>
 * Tag! game is simple children play where they are chasing each other, see http://en.wikipedia.org/wiki/Tag_%28game%29 
 * <p><p>
 * There is a 'map' directory containing 'DM-TagMap.ut2' that should be placed into 'UT2004/Maps' directory.
 * The map is simple "rectangle" with no internal walls / obstacles. Simple "rectangle" arena.
 * <p><p>
 * There is a class containing procedural description of the DM-TagMap, (see {@link TagMap}), thus you
 * can play with low-level movement command when implementing Tag! game behavior.
 * <p><p>
 * Note that the implementation is BLANK! You have to fill it for yourself.
 *
 * @author Jakub Gemrot aka Jimmy
 */
@AgentScoped
public class TagBot extends UT2004BotTagController {

	private static int number = 0;
	
	private TagEvents tagEvents;	
	
	private UnrealId tagBotId;
        
        private int gCounter;
        Player victim; // specifikuje obet, co byla vybrana v rezimu hledani 
	Player catcher; // specifiuje bota, od ktereho se ma utikat.	
    /**
     * Here we can modify initializing command for our bot.
     *
     * @return
     */
    @Override
    public Initialize getInitializeCommand() {
        return new Initialize().setName("Ivan_Krasicenko" + (++number));
        
    }

    /**
     * The bot is initialized in the environment - a physical representation of the bot is present in the game.
     *
     * @param config information about configuration
     * @param init information about configuration
     */
    @SuppressWarnings("unchecked")
    @Override
    public void botInitialized(GameInfo gameInfo, ConfigChange config, InitedMessage init) {
    	tagEvents = new TagEvents(getWorldView()) {
    		public void tagPlayerStatusChanged(TagPlayerStatusChanged event) {
    			if (event.getTagStatus()) {
    				tagBotId = event.getBotId();
    			}
    		}
    	};
    	tagEvents.enableTagEvents();
    }

    /**
     * The bot is initialized in the environment - a physical representation of
     * the bot is present in the game.
     *
     * @param config information about configuration
     * @param init information about configuration
     */
    @Override
    public void botFirstSpawn(GameInfo gameInfo, ConfigChange config, InitedMessage init, Self self) {
    }

    /**
     * This method is called only once right before actual logic() method is
     * called for the first time.
     */
    @Override
    public void beforeFirstLogic() {
    }

    /**
     * Main method that controls the bot - makes decisions what to do next. It
     * is called iteratively by Pogamut engine every time a synchronous batch
     * from the environment is received. This is usually 4 times per second - it
     * is affected by visionTime variable, that can be adjusted in GameBots ini
     * file in UT2004/System folder.
     */
    @Override
    public void logic() {
        // mark that another logic iteration has began
     //   log.info("--- Logic iteration ---");
       
        if (!tag.isGameRunning()) {
        	log.info("TAG GAME NOT RUNNING YET...");
        	move.stopMovement();
        	return;
        }        
        
        if (gCounter > 0) { 
            gCounter--;
            move.turnHorizontal(90); 
            return; 
        }
        
        
        // FIRST OF ALL we have to check in what "high-level" state of the game we're in
        
        // player, who has the tag ... can be null at the beginning of the game
        
        //players.getp
        if (tag.hasTag()) {
            catcher = null;
            // you HAVE THE TAG, chase other players ... do not chase players that are immune to you!	
            //log.info("I'm having the tag.");
            {
                     
            if (victim == null){
                //gCounter = 5; // Zapnu rotovani na 5 tiku
                //Vybiram obet
                Player recentPlayer = players.getNearestPlayer(10000);
                if (recentPlayer != null && !tag.isImmuneToMe(recentPlayer)){
                    victim = recentPlayer;
                } else {
                    Iterator<Player> plIt = players.getPlayers().values().iterator();
                   
                    while(plIt.hasNext()){
                        Player p = plIt.next();
                        if (p != null && !tag.isImmuneToMe(p)){
                            victim = p;
                        }
                    }
                }
            
            } else {
                // homim se za obeti.
                
                if (victim.getLocation() != null){
                
                    
                    //vector kam chci smerovat
                    Location vect = genCatchVect(victim.getLocation(),info.getLocation());
                    // pozice, kam chci jit
                    Location wantGo = Location.add(info.getLocation(),vect);
                    // jdu na pozici, a divam se na obet
                    move.strafeTo(wantGo, victim);



            } else {
                    // ROTUJI, abych nasel pozici obeti
                    gCounter = 5;
                }
                
                
            }
            
            }
            
            
        } else { 
            victim = null;
        if (tag.isMeImmune(tagBotId)) {
        	// you DO NOT HAVE the TAG and you are IMMUNE to the player who has the tag
        	//log.info("I'm immune to tag-holder.");
        } else {
        	// you DO NOT HAVE the TAG, you have to RUN AWAY from player who has the tag        	
        	//log.info("I can be tagged by tag-holder.");        	
        	
                if (catcher != null) {
        	// player having the tag was seen
                    if (players.getPlayer(tagBotId) != null)
                        {catcher = players.getPlayer(tagBotId);}
                    if (catcher.getLocation() != null){   
                        Location vect = genRunVect(catcher.getLocation(), info.getLocation());
                        Location wantGo = Location.add(vect,info.getLocation());     
                        move.strafeTo(wantGo, catcher);
                    } else {
                        gCounter = 6;
                        log.info("TAGGED LOCTAION NULL");
                    }
        	} else {
        		// player having the tag was not seen yet, try to look around
        		gCounter = 6; 
                        catcher = players.getPlayer(tagBotId);

                       log.info("I HAVE NOT SEEN the tag-holder YET.");
                  
                       
                       
                       
        	}
        }
        }
    }

    public static void main(String args[]) throws PogamutException {
    	// START 4 AGENTS FIRST
        new UT2004BotRunner(TagBot.class, "TagBot").setMain(false).setLogLevel(Level.WARNING).startAgents(1);
        
        // START UT2004TagServer
        //UT2004TagServerModule module = new UT2004TagServerModule();
        //UT2004ServerFactory factory = new UT2004ServerFactory(module);
        //UT2004ServerRunner serverRunner = new UT2004ServerRunner(factory);
        
        //UT2004TagServer server = (UT2004TagServer) serverRunner.setMain(false).startAgent();

        // START THE TAG GAME
        //server.startGame(5 * 60, 10);
    }
    
    
   Location genCatchVect(Location catcher, Location victim ) {


        Location answ = new Location(0, 0);
        double vahaConst  = 400;
        double velikostVektoru = 100;
         
        //Vektor catchera
        Location vectFromBot = Location.add(catcher, victim.invert()).getNormalized().scale(500);
        
        
        double distance = 0;
        double vaha = 0;
       
        answ = Location.add(answ, vectFromBot);       
  
        // vektor zdi EAST
        distance = TagMap.getWallDistance(TagMap.Direction.EAST, victim);
        vaha = vahaConst/distance;
        answ = Location.add(answ,new Location(-velikostVektoru,0,0).scale(vaha));
        
        // vektor zdi North
        distance =  TagMap.getWallDistance(TagMap.Direction.NORTH, victim);
        vaha = vahaConst/distance;
        answ = Location.add(answ,new Location(0,velikostVektoru,0).scale(vaha));
  
        // vektor zdi South
        distance = TagMap.getWallDistance(TagMap.Direction.SOUTH, victim);
        vaha = vahaConst/distance;
      
        answ = Location.add(answ,new Location(0,-velikostVektoru,0).scale(vaha));
        
        // vektor zdi West
        distance = TagMap.getWallDistance(TagMap.Direction.WEST, victim);
        vaha = vahaConst/distance;
        answ = Location.add(answ,new Location(velikostVektoru,0,0).scale(vaha));
        
        return answ;
     
  
    }
    
   
         Location genRunVect(Location tagBot, Location botPosition ) {
    
        
               
        Location answ = new Location(0, 0);
        double vahaConst  = 400;
        double vahaConstZdi = 400;
        double velikostVektoru = 100;
        double sumaVah = 0;

         
         
        //Vektor tagbota 
        Location vectFromBot = Location.add(botPosition, tagBot.invert()).getNormalized().scale(100);
        
        double distance = Location.getDistance(botPosition, tagBot);
        double vaha = vahaConst/distance;
        sumaVah += vaha;
        
        answ = Location.add(answ, vectFromBot.scale(vaha));       
        
        // vektor zdi EAST
        distance = TagMap.getWallDistance(TagMap.Direction.EAST, botPosition);
        vaha = vahaConstZdi/distance;
        sumaVah += vaha;
        answ = Location.add(answ,new Location(-velikostVektoru,0,0).scale(vaha));
              
        // vektor zdi North
        distance =  TagMap.getWallDistance(TagMap.Direction.NORTH, botPosition);
        vaha = vahaConstZdi/distance;
        sumaVah += vaha;
        answ = Location.add(answ,new Location(0,velikostVektoru,0).scale(vaha));
   
           
        // vektor zdi South
        distance = TagMap.getWallDistance(TagMap.Direction.SOUTH, botPosition);
        vaha = vahaConstZdi/distance;
        sumaVah += vaha;
        answ = Location.add(answ,new Location(0,-velikostVektoru,0).scale(vaha));
        
        
        // vektor zdi West
        distance = TagMap.getWallDistance(TagMap.Direction.WEST, botPosition);
        vaha = vahaConstZdi/distance;
        sumaVah += vaha;
        answ = Location.add(answ,new Location(velikostVektoru,0,0).scale(vaha));
        
        sumaVah = sumaVah/5*velikostVektoru;
        double silaVektoru = Math.abs(answ.getX())+  Math.abs(answ.getY());
        
        if (silaVektoru * 2 >  sumaVah){
            answ.scale(500);
        }
        
        return answ;
        
   
    }
    
}
